// gpio.h 2013-01-30 00:00
//
// (c) 2013 Markus Weber, Nuernberg
// GNU GPL v2 (see Readme.txt)
//
// This source code file adds general purpose i/o functions
// to USBasp version "guloprog" for ATtiny85 microcontroller.
//
// Include this file in USBasp's main.c directly before function
// definition of usbFunctionSetup() and place three function calls
// at these positions:
// if(data[1]>='A' && data[1]<='D') len= gpio_control(); else
//                      in function usbFunctionSetup(), right before
//                      this line: if (data[1] == USBASP_FUNC_CONNECT)
// gpio_ini():          initialization, before entering main loop
// gpio_main():         somewhere in main loop

//#include <inttypes.h>
//#include <stdlib.h>
//#include <string.h>
//#include <stdio.h>
//#include <ctype.h>
//#include <unistd.h>
//#include <signal.h>
//#include <usb.h>
typedef enum {false= 0,true= 1} bool;

// i/o pin A: PB4 (pin 3, no resistor)
// i/o pin B: PB3 (pin 2)
// i/o pin C: PB5 (pin 1, weak output)
// i/o pin D: PB0 (pin 5, no ADC)
#define GPIO_APIN 4
#define GPIO_BPIN 3
#define GPIO_CPIN 5
#define GPIO_DPIN 0
int8_t gpio_val[6];  // new value (0..100, -1, -2)
uint8_t gpio_adc_low[6];  // ADC result, low byte
uint8_t gpio_adc_high[6];  // ADC result, high byte
int8_t gpio_old_val,gpio_new_val;
  // for synchronisation of the procedures gpio_control() and gpio_min()



static inline uint8_t gpio_control() {
  // process a USB control message which has been received from host;
  // data[1]: i/o pin ('A'..'D'), must be in this range;
  // data[2]: control value for this pin;
  //          0: set pin to Low;
  //          1..99: activate PWM for this pin, 1..99 % High;
  //          100: set pin to High;
  //          -1: switch pin to input;
  //          -2: switch pin to input, activate pull-up resistor;
  //          -6: just get input values for this pin;
  // return: 4;
  // in case data[2]==-6, the replyBuffer[] is set to these values:
  // replyBuffer[0]: 0: pin is Low; 1: pin is High;
  // replyBuffer[1]: 0;
  // replyBuffer[2]: ADC value of the requested pin, low byte
  // replyBuffer[3]: ADC value of the requested pin, high byte
  uint8_t iopin;
  int8_t newval;

  iopin= data[1]-'A';
  newval= data[2];
  if(newval==-6) {  // no output, just get input values
    if(iopin==0) {
      replyBuffer[0]= PINB & (1<<GPIO_APIN);
      replyBuffer[2]= gpio_adc_low[GPIO_APIN];
      replyBuffer[3]= gpio_adc_high[GPIO_APIN];
      }
    else if(--iopin==0) {
      replyBuffer[0]= PINB & (1<<GPIO_BPIN);
      replyBuffer[2]= gpio_adc_low[GPIO_BPIN];
      replyBuffer[3]= gpio_adc_high[GPIO_BPIN];
      }
    else if(--iopin==0) {
      replyBuffer[0]= PINB & (1<<GPIO_CPIN);
      replyBuffer[2]= gpio_adc_low[GPIO_CPIN];
      replyBuffer[3]= gpio_adc_high[GPIO_CPIN];
      }
    else {
      replyBuffer[0]= PINB & (1<<GPIO_DPIN);
      // fake ADC values since i/o pin D hast no ADC
      if(replyBuffer[0]==0) {  // Low
        replyBuffer[2]= 0;
        replyBuffer[3]= 0;
        }
      else {  // High
        replyBuffer[2]= 0xff;
        replyBuffer[3]= 3;
        }
      }
    replyBuffer[1]= 0;
    }  // no output, just get input values
  else {  // store switch command
    if(iopin==0)
      gpio_val[GPIO_APIN]= newval;
    else if(--iopin==0)
      gpio_val[GPIO_BPIN]= newval;
    else if(--iopin==0)
      gpio_val[GPIO_CPIN]= newval;
    else
      gpio_val[GPIO_DPIN]= newval;
    gpio_new_val++;  // inform gpio_main() about the change
    }  // store switch command
  return 4;
  }  // gpio_control()



static inline void gpio_ini() {
  // initialize general purpose i/o funtions;
  gpio_val[GPIO_APIN]= -2;
  uint8_t gpio_adc_low[GPIO_APIN]= 0;
  uint8_t gpio_adc_high[GPIO_APIN]= 0;
// the previous section must be copied for i/o pins B, C, D ! ,,,,,
  gpio_old_val= gpio_new_val= 0;
  TCNT1= 0;  // Timer 1 starts counting at 0
  OCR1C= 99;  // maximum value for Timer 1
  TCCR1= (1<<CTC1)|(1<<CS13)|(0<<CS12)|(1<<CS11)|(1<<CS10);
    // start Timer 1 with prescaler 1024; -> 16500 Hz
    // set CTC1 (Clear Timer/Counter on Compare Match)





  }  // gpio_ini()



static inline void gpio_main() {
  // care about PWM output (soft PWM) and read ADC results;




  // care about i/o pin states
  if(gpio_new_val!=gpio_old_val) {  // at least one value has been changed
    int8_t newval;

    gpio_old_val++;  // remember that we have reacted
    // set new values to i/o pins;
    // gpio_val[]: values which represent the i/o pin states;
    //             0: Low; 1: High; 1..9: PWM;
    //             -1: input; -2: input + pull-up;
    newval= gpio_out[GPIO_APIN];
    if(newval<0) {  // switch pin to input, set pull-up if requested
      DDRB&= ~(1<<GPIO_APIN);
      if(++newval==0) PORTB&= ~(1<<GPIO_APIN);
      else PORTB|= (1<<GPIO_APIN);
      }
    else {  // switch to output
      if(newval==0)  // switch to Low
        PORTB&= ~(1<<GPIO_APIN);
      else if(newval==100)  // switch to High
        PORTB|= (1<<GPIO_APIN);
      // else PWM, do not switch, just wait for PWM cycle
      DDRB|= (1<<GPIO_APIN);  // switch direction to output
      }  // switch to output
// the previous section must be copied for i/o pins B, C, D ! ,,,,,
    }  // at least one value has been changed

18,51,84
  }  // gpio_main()


